# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Printer for formatting the 'gcloud run intergrations list' command.

The integrations list command returns a dict with a single key.
The value is a list of dicts where each entry will be formatted as a row in the
resulting table that will be displayed to the user.

input:
  record: {
    'output': [
        {'name': 'integration1', 'type': 'redis', '
        services': 'svc1,svc2', 'latestDeployment': FAILED},
        {'name': 'integration2', 'type': 'redis', '
        services': 'svc3,svc4', 'latestDeployment': SUCCESS},
    ]
  }

output:
    INTEGRATION     TYPE       SERVICE
  X integration1    redis      svc1,svc2
  ✔ integration2    redis      svc3,svc4
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.command_lib.run.integrations import deployment_states
from googlecloudsdk.command_lib.run.integrations.formatters import base_formatter
from googlecloudsdk.core.resource import custom_printer_base as cp

PRINTER_FORMAT = 'IntegrationsList'
RECORD_KEY = 'OUTPUT'


class Row(object):
  """Represents the fields that will be used in the output of the table.

  Having a single class that has the expected values here is better than passing
  around a dict as the keys could mispelled or changed in one place.
  """

  def __init__(self, integration_name, integration_type,
               services, latest_deployment_status):
    self.integration_name = integration_name
    self.integration_type = integration_type
    self.services = services
    self.latest_deployment_status = latest_deployment_status

  def __eq__(self, other):
    return (self.integration_name == other.integration_name and
            self.integration_type == other.integration_type and
            self.services == other.services and
            self.latest_deployment_status == other.latest_deployment_status
           )


# TODO(b/217741829): Limit outputted services to 3.
class IntegrationListPrinter(cp.CustomPrinterBase):
  """Prints the integrations list output in a custom human readable format."""

  def Transform(self, record):
    """Transforms the given record into a structured table.

    Args:
      record: dict, has a single key. Values are list[dict] where each record
        in the list will be a separate row in the output.

    Returns:
      Formatted table that is displayed on output to the console.
    """
    rows = record[RECORD_KEY]
    rows = [(_GetSymbolFromDeploymentStatus(row.latest_deployment_status),
             row.integration_name, row.integration_type, row.services)
            for row in rows
           ]

    # the empty column is for the latest deployment status.
    cols = [('', 'INTEGRATION', 'TYPE', 'SERVICE')]
    return cp.Table(cols + rows)


def _GetSymbolFromDeploymentStatus(status):
  """Gets a symbol based on the latest deployment status.

  If a deployment cannot be found or the deployment is not in a 'SUCCEEDED',
  'FAILED', or 'IN_PROGRESS' state, then it should be reported as 'FAILED'.

  This would be true for integrations where the deployment never kicked off
  due to a failure.

  Args:
    status: The latest deployment status.

  Returns:
    str, the symbol to be placed in front of the integration name.
  """
  status_to_symbol = {
      deployment_states.SUCCEEDED:
          base_formatter.GetSymbol(base_formatter.SUCCESS),
      deployment_states.FAILED:
          base_formatter.GetSymbol(base_formatter.FAILED),
      deployment_states.IN_PROGRESS:
          base_formatter.GetSymbol(base_formatter.UPDATING),
  }

  return status_to_symbol.get(status,
                              base_formatter.GetSymbol(
                                  base_formatter.FAILED
                                  )
                              )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Lightweight websocket for IAP."""

# See https://datatracker.ietf.org/doc/html/rfc6455 for the protocol used.
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import errno
import select
import socket
import ssl
import struct

from googlecloudsdk.core.util import platforms
import six
import websocket._abnf as websocket_frame_utils
import websocket._exceptions as websocket_exceptions
import websocket._handshake as websocket_handshake
import websocket._http as websocket_http_utils
import websocket._utils as websocket_utils

WEBSOCKET_RETRY_TIMEOUT_SECS = 3
WEBSOCKET_MAX_ATTEMPTS = 3
CLOSE_STATUS_NORMAL = 1000
VALID_OPCODES = [
    websocket_frame_utils.ABNF.OPCODE_CLOSE,
    websocket_frame_utils.ABNF.OPCODE_BINARY
]


class SockOpt(object):
  """Class that represents the options for the underlying socket library."""

  def __init__(self, sslopt):
    if sslopt is None:
      sslopt = {}
    # Timeout and sockopt are required for the underlying library but we don't
    # use it.
    self.timeout = None
    self.sockopt = []
    self.sslopt = sslopt


class _FrameBuffer(object):
  """Class that represents one single frame sent or received by the websocket."""

  def __init__(self, recv_fn):
    self.recv = recv_fn

  def _recv_header(self):
    """Parse the header from the message."""
    header = self.recv(2)
    b1 = header[0]

    if six.PY2:
      b1 = ord(b1)

    fin = b1 >> 7 & 1
    rsv1 = b1 >> 6 & 1
    rsv2 = b1 >> 5 & 1
    rsv3 = b1 >> 4 & 1
    opcode = b1 & 0xf
    b2 = header[1]

    if six.PY2:
      b2 = ord(b2)

    has_mask = b2 >> 7 & 1
    length_bits = b2 & 0x7f

    return (fin, rsv1, rsv2, rsv3, opcode, has_mask, length_bits)

  def _recv_length(self, bits):
    """Parse the length from the message."""
    length_bits = bits & 0x7f
    if length_bits == 0x7e:
      v = self.recv(2)
      return struct.unpack("!H", v)[0]
    elif length_bits == 0x7f:
      v = self.recv(8)
      return struct.unpack("!Q", v)[0]
    else:
      return length_bits

  def recv_frame(self):
    """Receives the whole frame."""
    # Receives the header.
    (fin, rsv1, rsv2, rsv3, opcode, has_mask, length_bits) = self._recv_header()
    if has_mask == 1:
      raise Exception("Server should not mask the response")
    # Receives the frame length.
    length = self._recv_length(length_bits)
    # Receives the payload.
    payload = self.recv(length)

    return websocket_frame_utils.ABNF(fin, rsv1, rsv2, rsv3, opcode, has_mask,
                                      payload)


class IapLightWeightWebsocket(object):
  """Lightweight websocket created to send and receive data as fast as possible.

     This websocket implements rfc6455
  """

  def __init__(self,
               url,
               header,
               on_data,
               on_close,
               on_error,
               subprotocols,
               sock=None):
    self.url = url
    self.on_data = on_data
    self.on_close = on_close
    self.on_error = on_error
    # Sock is an already connected socket
    self.sock = sock
    # Used to store frame data during recv.
    self.frame_buffer = _FrameBuffer(self._recv_bytes)
    self.connected = False
    # Used to fix the mask key used. This is important for unit tests so we
    # don't use a random mask everytime.
    self.get_mask_key = None
    self.subprotocols = subprotocols
    self.header = header

  def recv(self):
    """Receives data from the server."""
    if not self.connected or not self.sock:
      raise websocket_exceptions.WebSocketConnectionClosedException(
          "Connection closed while receiving data.")
    # We will call recv_frame, which does the recv + parsing of the message
    return self.frame_buffer.recv_frame()

  def send(self, data, opcode):
    """Sends data to the server."""
    if opcode not in VALID_OPCODES:
      raise ValueError("Invalid opcode")
    # Create the frame, fin indicates the end, so we are sending only one frame.
    # Rsv1,rsv2 and rsv3 should be 0, they are extra flags but our server
    # doesn't interpret them. Mask is a fixed value that indicates if we want to
    # mask the data
    frame_data = websocket_frame_utils.ABNF(
        fin=1, rsv1=0, rsv2=0, rsv3=0, opcode=opcode, mask=1, data=data)

    if self.get_mask_key:
      frame_data.get_mask_key = self.get_mask_key
    frame_data = frame_data.format()
    # We will try to send up to WEBSOCKET_MAX_ATTEMPTS in case the
    # exception is not fatal.
    for attempt in range(1, WEBSOCKET_MAX_ATTEMPTS + 1):
      try:
        if not self.connected or not self.sock:
          raise websocket_exceptions.WebSocketConnectionClosedException(
              "Connection closed while sending data.")
        bytes_sent = self.sock.send(frame_data)
        # No bytes sent means the socket is closed.
        if not bytes_sent:
          raise websocket_exceptions.WebSocketConnectionClosedException(
              "Connection closed while sending data.")

        # If we failed to send all the data, throw error.
        if len(frame_data) != bytes_sent:
          raise Exception("Packet was not sent in it's entirety")
        return bytes_sent
      except Exception as e:  # pylint: disable=broad-except
        self._throw_or_wait_for_retry(attempt=attempt, exception=e)

  def close(self, close_code=CLOSE_STATUS_NORMAL, close_message=six.b("")):
    """Closes the connection."""
    if self.connected and self.sock:
      try:
        self.send_close(close_code, close_message)
        self.sock.close()
        self.sock = None
        self.connected = False
      except (websocket_exceptions.WebSocketConnectionClosedException,
              socket.error) as e:
        # We don't throw the error if it's a socket closed exception,
        # this is to avoid the caller breaking due to calling close too much
        if not self._is_closed_connection_exception(e):
          raise

  def send_close(self, close_code=CLOSE_STATUS_NORMAL, close_message=six.b("")):
    """Sends a close message to the server but don't close."""
    if self.connected:
      # six.b() returns a native str. In python2 it is an ascii encoded string,
      # which causes encoding problems bellow when we are trying to combine with
      # struct.pack, which returns a latin-1 encoded string. For python 3 we
      # don't have to worry about it as native str are latin-1 encoded.
      if six.PY2:
        close_message = close_message.encode("latin-1")
      try:
        self.send(
            struct.pack("!H", close_code) + close_message,
            websocket_frame_utils.ABNF.OPCODE_CLOSE)
      # Trying to send a close when the websocket is already closed will result
      # in two different errors depending on which stage of the connection we
      # are.
      except (websocket_exceptions.WebSocketConnectionClosedException, OSError,
              socket.error, ssl.SSLError) as e:
        # We don't throw the error if it's a socket closed exception, as we were
        # trying to close it anyways.
        if not self._is_closed_connection_exception(e):
          raise

  def run_forever(self, sslopt, **options):
    """Main method that will stay running while the connection is open."""
    try:
      options.update({"header": self.header})
      options.update({"subprotocols": self.subprotocols})
      self._connect(sslopt, **options)
      while self.connected:
        # If the socket is already closed, we throw
        if self.sock.fileno() == -1:
          raise websocket_exceptions.WebSocketConnectionClosedException(
              "Connection closed while receiving data.")
        # Wait for websocket to be ready so we don't use too much cpu looping
        # forever.
        self._wait_for_socket_to_ready(timeout=WEBSOCKET_RETRY_TIMEOUT_SECS)
        frame = self.recv()
        if frame.opcode == websocket_frame_utils.ABNF.OPCODE_CLOSE:
          close_args = self._get_close_args(frame.data)
          self.close()
          self.on_close(*close_args)
        else:
          self.on_data(frame.data, frame.opcode)
    except KeyboardInterrupt as e:
      self.close()
      self.on_close(close_code=None, close_message=None)
      raise e
    except Exception as e:  # pylint: disable=broad-except
      self.close()
      self.on_error(e)
      error_code = websocket_utils.extract_error_code(e)
      message = websocket_utils.extract_err_message(e)
      self.on_close(error_code, message)

  def _recv_bytes(self, buffersize):
    """Internal implementation of recv called by recv_frame."""
    view = memoryview(bytearray(buffersize))
    total_bytes_read = 0
    # We try to read WEBSOCKET_MAX_ATTEMPTS times given it's not a fatal
    # exception.
    for attempt in range(1, WEBSOCKET_MAX_ATTEMPTS + 1):
      try:
        # We might not have "buffersize" bytes ready to read, but we need to
        # read "buffersize" as the caller (recv_frame) always sets buffersize
        # with the exact amount it needs, so in that case we loop until we read
        # all.
        while total_bytes_read < buffersize:
          # We read how much is still left.
          bytes_received = self.sock.recv_into(view[total_bytes_read:],
                                               buffersize - total_bytes_read)
          # If we receive 0 bytes it means connection closed, regardless if we
          # have data we read before in "total_bytes_received", we should close
          # the connection.
          if bytes_received == 0:
            self.close()
            raise websocket_exceptions.WebSocketConnectionClosedException(
                "Connection closed while receiving data.")
          total_bytes_read += bytes_received
        return view.tobytes()
      except Exception as e:  # pylint: disable=broad-except
        self._throw_or_wait_for_retry(attempt=attempt, exception=e)

  def _set_mask_key(self, mask_key):
    self.get_mask_key = mask_key

  def _connect(self, ssl_opt, **options):
    """Connect method, doesn't follow redirects."""
    proxy = websocket_http_utils.proxy_info(**options)
    sockopt = SockOpt(ssl_opt)
    # We don't need to connect and do the handshake if the caller already
    # specified a websocket.
    if self.sock:
      hostname, port, resource, _ = websocket_http_utils.parse_url(self.url)
      addrs = (hostname, port, resource)
    else:
      self.sock, addrs = websocket_http_utils.connect(self.url, sockopt, proxy,
                                                      None)
      websocket_handshake.handshake(self.sock, *addrs, **options)
    self.connected = True
    return addrs

  def _throw_on_non_retriable_exception(self, e):
    """Decides if we throw or if we ignore the exception because it's retriable."""

    if self._is_closed_connection_exception(e):
      raise websocket_exceptions.WebSocketConnectionClosedException(
          "Connection closed while waiting for retry.")
    if e is ssl.SSLError:
      # SSL_ERROR_WANT_WRITE can happen if the socket gives EAGAIN or
      # EWOULDBLOCK during the SSL handshake, which is a transient error.
      if e.args[0] != ssl.SSL_ERROR_WANT_WRITE:
        raise e
    elif e is socket.error:
      error_code = websocket_utils.extract_error_code(e)
      if error_code is None:
        raise e
      # EWOULDBLOCK = sender buffer is full, EAGAIN = transitory error.
      if error_code != errno.EAGAIN or error_code != errno.EWOULDBLOCK:
        raise e

  def _throw_or_wait_for_retry(self, attempt, exception):
    """Wait for the websocket to be ready we don't retry too much too quick."""
    self._throw_on_non_retriable_exception(exception)
    # We want to wait some time before we retry, just to make sure the
    # buffer is emptying, but if the socket gets ready before that then we
    # send.
    if (attempt < WEBSOCKET_MAX_ATTEMPTS and self.sock and
        self.sock.fileno() != -1):
      self._wait_for_socket_to_ready(WEBSOCKET_RETRY_TIMEOUT_SECS)
    else:
      raise exception

  def _wait_for_socket_to_ready(self, timeout):
    """Wait for socket to be ready and treat some special errors cases."""
    try:
      _ = select.select([self.sock], (), (), timeout)
    except TypeError as e:
      message = websocket_utils.extract_err_message(e)
      # There's a possibility that the socket gets transitioned to an invalid
      # state (i.e NoneType) while we are waiting for the select,
      # in which case we will throw the websocket closed error
      if isinstance(message,
                    str) and "arguments 1-3 must be sequences" in message:
        raise websocket_exceptions.WebSocketConnectionClosedException(
            "Connection closed while waiting for socket to ready.")
      raise
    # select.error is the equivalent of OSError in python 2.7
    except (OSError, select.error) as e:
      # For windows, mocking the socket will throw on this select as it only
      # support sockets (for linux we bypass that by implementing fileno). The
      # error below is "An operation was attempted on something that is not a
      # socket", which will never happen unless this is a socket based on a
      # file or a mocked socket, in which case we just let execution continue.
      if not platforms.OperatingSystem.IsWindows():
        raise
      if e is OSError  and e.winerror != 10038:
        raise
      if e is select.error and e.errno != errno.ENOTSOCK:
        raise

  def _is_closed_connection_exception(self, exception):
    """Method to identify if the exception is of closed connection type."""
    if exception is websocket_exceptions.WebSocketConnectionClosedException:
      return True
    elif exception is OSError and exception.errno == errno.EBADF:
      # Errno.EBADF means the file descriptor was already closed (common error
      # when interacting with already closed websockets).
      return True
    elif exception is ssl.SSLError:
      # SSL_ERROR_EOF can happen if the socket gives EBADF during the SSL
      # handshake, which means the socket is closed.
      if exception.args[0] == ssl.SSL_ERROR_EOF:
        return True
    else:
      error_code = websocket_utils.extract_error_code(exception)
      # ENOTCONN == transport endpoint not connected.
      # EPIPE == broken pipe, writing to the socket when the other end
      # terminated the connection
      if error_code == errno.ENOTCONN or error_code == errno.EPIPE:
        return True
    return False

  def _get_close_args(self, data):
    if data and len(data) >= 2:
      code = 256 * six.byte2int(data[0:1]) + six.byte2int(data[1:2])
      reason = data[2:].decode("utf-8")
      return [code, reason]



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This typekit contains functions implemented by the CloudSQL integration.

The base functions from the TypeKit class can be overridden here if more
functionality is needed.  For now, the CloudSQL integration almost entirely
works out of the box with what works in the base class.

TODO(b/191327853):
Once private IP is supported then this class needs to be updated.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.command_lib.run.integrations.typekits import base


class CloudSqlTypeKit(base.TypeKit):
  """The Cloud SQL integration typekit."""

  def UpdateResourceConfig(self, parameters, resource_config):
    """Updates the existing resource config with the parameters provided.

    Args:
      parameters: dict, user provided parameters from the command.
      resource_config: dict, resource config associated with the integration.
    """
    settings = resource_config.setdefault('settings', {})
    if 'tier' in parameters:
      settings['tier'] = parameters['tier']
    if 'version' in parameters:
      resource_config['version'] = parameters['version']




# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List templates command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.fleet.policycontroller import status_api_utils
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.core import properties
from surface.container.fleet.policycontroller import templates as poco_templates


@calliope_base.Hidden
@calliope_base.ReleaseTracks(calliope_base.ReleaseTrack.ALPHA)
class List(calliope_base.ListCommand):
  """List Policy Controller constraint templates.

  ## EXAMPLES

  To list all Policy Controller constraint templates across the Fleet:

      $ {command}
  """

  @staticmethod
  def Args(parser):
    calliope_base.URI_FLAG.RemoveFromParser(parser)
    calliope_base.PAGE_SIZE_FLAG.SetDefault(parser, 100)
    parser.add_argument(
        '--verbose',
        action='store_true',
        help='Include extended information about constraint templates.',
        default=False)

  def Run(self, args):
    calliope_base.EnableUserProjectQuota()

    project_id = properties.VALUES.core.project.Get(required=True)

    client = status_api_utils.GetClientInstance(
        self.ReleaseTrack())
    messages = status_api_utils.GetMessagesModule(
        self.ReleaseTrack())

    formatted_templates = {}

    if args.verbose:
      return poco_templates.list_membership_templates(project_id, messages,
                                                      client)

    # Return fleet templates if verbose flag is not given.
    request = messages.AnthospolicycontrollerstatusPaProjectsFleetConstraintTemplatesListRequest(
        parent='projects/' + project_id)
    response = client.projects_fleetConstraintTemplates.List(request)

    for template in response.fleetConstraintTemplates:
      formatted_template = {
          'name': template.ref.name,
          'constraints': template.numConstraints or 0,
          'memberships': template.numMemberships or 0
      }
      formatted_templates[template.ref.name] = formatted_template

    return [v for _, v in sorted(formatted_templates.items())]



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Formatting strings for Artifact Registry commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

BUILD_GIT_SHA_FORMAT = ("BUILD_DETAILS.buildDetails.provenance."
                        "sourceProvenance.context.cloudRepo.revisionId"
                        ".notnull().list().slice(:8).join(''):optional:label"
                        "=GIT_SHA")

BUILD_FORMAT = ("BUILD_DETAILS.buildDetails.provenance.id.notnull().list()"
                ":optional:label=BUILD")

VULNERABILITY_FORMAT = "vuln_counts.list():optional:label=VULNERABILITIES"

IMAGE_BASIS_FORMAT = ("IMAGE_BASIS.derivedImage.sort(distance).map()"
                      ".extract(baseResourceUrl).slice(:1).map().list().list()"
                      ".split('//').slice(1:).list().split('@').slice(:1)"
                      ".list():optional:label=FROM")

DISCOVERY_FORMAT = ("DISCOVERY[0].discovered.analysisStatus:optional:label"
                    "=VULNERABILITY_SCAN_STATUS")

CONTAINER_ANALYSIS_METADATA_FORMAT = """
  {},
  {},
  {},
  {},
  {}
""".format(BUILD_GIT_SHA_FORMAT, VULNERABILITY_FORMAT, IMAGE_BASIS_FORMAT,
           BUILD_FORMAT, DISCOVERY_FORMAT)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'ids endpoints delete' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import datetime
from googlecloudsdk.calliope import base
from googlecloudsdk.api_lib.ids import ids_api
from googlecloudsdk.command_lib.ids import flags
from googlecloudsdk.core import resources

DETAILED_HELP = {
    'DESCRIPTION':
        """
          Delete a Cloud IDS endpoint.
        """,
    'EXAMPLES':
        """
          To delete an endpoint called `my-ep` in project `my-project`
          and zone `us-central1-a`, run:

          $ {command} my-ep --project=my-project --zone=us-central1-a

          OR

          $ {command} projects/myproject/locations/us-central1-a/endpoints/my-ep

    """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Delete a Cloud IDS endpoint."""

  @staticmethod
  def Args(parser):
    flags.AddEndpointResource(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)
    flags.AddMaxWait(parser, '60m')  # default to 60 minutes wait.

  def Run(self, args):
    endpoint = args.CONCEPTS.endpoint.Parse()
    is_async = args.async_
    max_wait = datetime.timedelta(seconds=args.max_wait)

    client = ids_api.Client(self.ReleaseTrack())
    operation = client.DeleteEndpoint(endpoint.RelativeName())

    # Return the in-progress operation if async is requested.
    if is_async:
      # Delete operations have no format by default,
      # but here we want the operation metadata to be printed.
      if not args.IsSpecified('format'):
        args.format = 'default'
      return operation
    return client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message='waiting for endpoint [{}] to be deleted'.format(
            endpoint.RelativeName()),
        has_result=False,
        max_wait=max_wait)


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Artifact Registry files."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


class Files(base.Group):
  """Manage Artifact Registry files.

  ## EXAMPLES
  To list all files in the current project and `artifacts/repository` and
  `artifacts/location` properties are set,
  run:

      $ {command} list

  To list files under repository my-repo in the current project and location,
  run:

      $ {command} list --repository=my-repo
  """

  category = base.CI_CD_CATEGORY



"""Generated client library for certificatemanager version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.third_party.apis.certificatemanager.v1 import certificatemanager_v1_messages as messages


class CertificatemanagerV1(base_api.BaseApiClient):
  """Generated client library for service certificatemanager version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://certificatemanager.googleapis.com/'
  MTLS_BASE_URL = 'https://certificatemanager.mtls.googleapis.com/'

  _PACKAGE = 'certificatemanager'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'CertificatemanagerV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new certificatemanager handle."""
    url = url or self.BASE_URL
    super(CertificatemanagerV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_certificateIssuanceConfigs = self.ProjectsLocationsCertificateIssuanceConfigsService(self)
    self.projects_locations_certificateMaps_certificateMapEntries = self.ProjectsLocationsCertificateMapsCertificateMapEntriesService(self)
    self.projects_locations_certificateMaps = self.ProjectsLocationsCertificateMapsService(self)
    self.projects_locations_certificates = self.ProjectsLocationsCertificatesService(self)
    self.projects_locations_dnsAuthorizations = self.ProjectsLocationsDnsAuthorizationsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_trustConfigs = self.ProjectsLocationsTrustConfigsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsCertificateIssuanceConfigsService(base_api.BaseApiService):
    """Service class for the projects_locations_certificateIssuanceConfigs resource."""

    _NAME = 'projects_locations_certificateIssuanceConfigs'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsCertificateIssuanceConfigsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new CertificateIssuanceConfig in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateIssuanceConfigsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateIssuanceConfigs',
        http_method='POST',
        method_id='certificatemanager.projects.locations.certificateIssuanceConfigs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['certificateIssuanceConfigId'],
        relative_path='v1/{+parent}/certificateIssuanceConfigs',
        request_field='certificateIssuanceConfig',
        request_type_name='CertificatemanagerProjectsLocationsCertificateIssuanceConfigsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single CertificateIssuanceConfig.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateIssuanceConfigsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateIssuanceConfigs/{certificateIssuanceConfigsId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.certificateIssuanceConfigs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateIssuanceConfigsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single CertificateIssuanceConfig.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateIssuanceConfigsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CertificateIssuanceConfig) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateIssuanceConfigs/{certificateIssuanceConfigsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateIssuanceConfigs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateIssuanceConfigsGetRequest',
        response_type_name='CertificateIssuanceConfig',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists CertificateIssuanceConfigs in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateIssuanceConfigsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCertificateIssuanceConfigsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateIssuanceConfigs',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateIssuanceConfigs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/certificateIssuanceConfigs',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateIssuanceConfigsListRequest',
        response_type_name='ListCertificateIssuanceConfigsResponse',
        supports_download=False,
    )

  class ProjectsLocationsCertificateMapsCertificateMapEntriesService(base_api.BaseApiService):
    """Service class for the projects_locations_certificateMaps_certificateMapEntries resource."""

    _NAME = 'projects_locations_certificateMaps_certificateMapEntries'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsCertificateMapsCertificateMapEntriesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new CertificateMapEntry in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}/certificateMapEntries',
        http_method='POST',
        method_id='certificatemanager.projects.locations.certificateMaps.certificateMapEntries.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['certificateMapEntryId'],
        relative_path='v1/{+parent}/certificateMapEntries',
        request_field='certificateMapEntry',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single CertificateMapEntry.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}/certificateMapEntries/{certificateMapEntriesId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.certificateMaps.certificateMapEntries.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single CertificateMapEntry.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CertificateMapEntry) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}/certificateMapEntries/{certificateMapEntriesId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateMaps.certificateMapEntries.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesGetRequest',
        response_type_name='CertificateMapEntry',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists CertificateMapEntries in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCertificateMapEntriesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}/certificateMapEntries',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateMaps.certificateMapEntries.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/certificateMapEntries',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesListRequest',
        response_type_name='ListCertificateMapEntriesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a CertificateMapEntry.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}/certificateMapEntries/{certificateMapEntriesId}',
        http_method='PATCH',
        method_id='certificatemanager.projects.locations.certificateMaps.certificateMapEntries.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='certificateMapEntry',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCertificateMapEntriesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsCertificateMapsService(base_api.BaseApiService):
    """Service class for the projects_locations_certificateMaps resource."""

    _NAME = 'projects_locations_certificateMaps'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsCertificateMapsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new CertificateMap in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps',
        http_method='POST',
        method_id='certificatemanager.projects.locations.certificateMaps.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['certificateMapId'],
        relative_path='v1/{+parent}/certificateMaps',
        request_field='certificateMap',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single CertificateMap. A Certificate Map can't be deleted if it contains Certificate Map Entries. Remove all the entries from the map before calling this method.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.certificateMaps.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single CertificateMap.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (CertificateMap) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateMaps.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsGetRequest',
        response_type_name='CertificateMap',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists CertificateMaps in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCertificateMapsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificateMaps.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/certificateMaps',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsListRequest',
        response_type_name='ListCertificateMapsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a CertificateMap.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificateMapsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificateMaps/{certificateMapsId}',
        http_method='PATCH',
        method_id='certificatemanager.projects.locations.certificateMaps.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='certificateMap',
        request_type_name='CertificatemanagerProjectsLocationsCertificateMapsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsCertificatesService(base_api.BaseApiService):
    """Service class for the projects_locations_certificates resource."""

    _NAME = 'projects_locations_certificates'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsCertificatesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new Certificate in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificatesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificates',
        http_method='POST',
        method_id='certificatemanager.projects.locations.certificates.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['certificateId'],
        relative_path='v1/{+parent}/certificates',
        request_field='certificate',
        request_type_name='CertificatemanagerProjectsLocationsCertificatesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single Certificate.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificatesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificates/{certificatesId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.certificates.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificatesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single Certificate.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificatesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Certificate) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificates/{certificatesId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificates.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificatesGetRequest',
        response_type_name='Certificate',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists Certificates in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificatesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListCertificatesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificates',
        http_method='GET',
        method_id='certificatemanager.projects.locations.certificates.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/certificates',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsCertificatesListRequest',
        response_type_name='ListCertificatesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a Certificate.

      Args:
        request: (CertificatemanagerProjectsLocationsCertificatesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/certificates/{certificatesId}',
        http_method='PATCH',
        method_id='certificatemanager.projects.locations.certificates.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='certificate',
        request_type_name='CertificatemanagerProjectsLocationsCertificatesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsDnsAuthorizationsService(base_api.BaseApiService):
    """Service class for the projects_locations_dnsAuthorizations resource."""

    _NAME = 'projects_locations_dnsAuthorizations'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsDnsAuthorizationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new DnsAuthorization in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsDnsAuthorizationsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/dnsAuthorizations',
        http_method='POST',
        method_id='certificatemanager.projects.locations.dnsAuthorizations.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['dnsAuthorizationId'],
        relative_path='v1/{+parent}/dnsAuthorizations',
        request_field='dnsAuthorization',
        request_type_name='CertificatemanagerProjectsLocationsDnsAuthorizationsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single DnsAuthorization.

      Args:
        request: (CertificatemanagerProjectsLocationsDnsAuthorizationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/dnsAuthorizations/{dnsAuthorizationsId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.dnsAuthorizations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsDnsAuthorizationsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single DnsAuthorization.

      Args:
        request: (CertificatemanagerProjectsLocationsDnsAuthorizationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (DnsAuthorization) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/dnsAuthorizations/{dnsAuthorizationsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.dnsAuthorizations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsDnsAuthorizationsGetRequest',
        response_type_name='DnsAuthorization',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists DnsAuthorizations in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsDnsAuthorizationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListDnsAuthorizationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/dnsAuthorizations',
        http_method='GET',
        method_id='certificatemanager.projects.locations.dnsAuthorizations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/dnsAuthorizations',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsDnsAuthorizationsListRequest',
        response_type_name='ListDnsAuthorizationsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a DnsAuthorization.

      Args:
        request: (CertificatemanagerProjectsLocationsDnsAuthorizationsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/dnsAuthorizations/{dnsAuthorizationsId}',
        http_method='PATCH',
        method_id='certificatemanager.projects.locations.dnsAuthorizations.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='dnsAuthorization',
        request_type_name='CertificatemanagerProjectsLocationsDnsAuthorizationsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (CertificatemanagerProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='certificatemanager.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='CertificatemanagerProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (CertificatemanagerProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (CertificatemanagerProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (CertificatemanagerProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='certificatemanager.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsTrustConfigsService(base_api.BaseApiService):
    """Service class for the projects_locations_trustConfigs resource."""

    _NAME = 'projects_locations_trustConfigs'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsTrustConfigsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new TrustConfig in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsTrustConfigsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/trustConfigs',
        http_method='POST',
        method_id='certificatemanager.projects.locations.trustConfigs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['trustConfigId'],
        relative_path='v1/{+parent}/trustConfigs',
        request_field='trustConfig',
        request_type_name='CertificatemanagerProjectsLocationsTrustConfigsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single TrustConfig.

      Args:
        request: (CertificatemanagerProjectsLocationsTrustConfigsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/trustConfigs/{trustConfigsId}',
        http_method='DELETE',
        method_id='certificatemanager.projects.locations.trustConfigs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['etag'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsTrustConfigsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single TrustConfig.

      Args:
        request: (CertificatemanagerProjectsLocationsTrustConfigsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TrustConfig) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/trustConfigs/{trustConfigsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.trustConfigs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsTrustConfigsGetRequest',
        response_type_name='TrustConfig',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists TrustConfigs in a given project and location.

      Args:
        request: (CertificatemanagerProjectsLocationsTrustConfigsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListTrustConfigsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/trustConfigs',
        http_method='GET',
        method_id='certificatemanager.projects.locations.trustConfigs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1/{+parent}/trustConfigs',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsTrustConfigsListRequest',
        response_type_name='ListTrustConfigsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a TrustConfig.

      Args:
        request: (CertificatemanagerProjectsLocationsTrustConfigsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/trustConfigs/{trustConfigsId}',
        http_method='PATCH',
        method_id='certificatemanager.projects.locations.trustConfigs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1/{+name}',
        request_field='trustConfig',
        request_type_name='CertificatemanagerProjectsLocationsTrustConfigsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (CertificatemanagerProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='certificatemanager.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (CertificatemanagerProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='certificatemanager.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1/{+name}/locations',
        request_field='',
        request_type_name='CertificatemanagerProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(CertificatemanagerV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Redis formatter for Cloud Run Integrations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.command_lib.run.integrations.formatters import base_formatter
from googlecloudsdk.core.resource import custom_printer_base as cp

_REDIS_INSTANCE_TYPE = 'google_redis_instance'
_VPC_INSTANCE_TYPE = 'google_vpc_access_connector'


class RedisFormatter(base_formatter.BaseFormatter):
  """Format logics for redis integration."""

  def TransformConfig(self, record):
    """Print the config of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    res_config = record.get('config', {}).get('redis', {}).get('instance', {})

    labeled = [('Memory Size GB', res_config.get('memory-size-gb'))]
    if 'tier' in res_config:
      labeled.append(('Tier', res_config.get('tier')))
    if 'version' in res_config:
      labeled.append(('Version', res_config.get('version')))

    return cp.Labeled(labeled)

  def TransformComponentStatus(self, record):
    """Print the component status of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    resource_status = record.get('status', {})
    resources = resource_status.get('resourceComponentStatuses', {})
    redis = self._RedisFromResources(resources)
    vpc = self._VpcFromResources(resources)
    return cp.Labeled([
        cp.Lines([
            ('MemoryStore Redis ({})'.format(
                redis.get('name', ''))),
            cp.Labeled([
                ('Console link', redis.get('consoleLink', 'n/a')),
                ('Resource Status', redis.get('state', 'n/a')),
            ]),
        ]),
        cp.Lines([
            ('Serverless VPC Connector ({})'.format(vpc.get('name', ''))),
            cp.Labeled([
                ('Console link', vpc.get('consoleLink', 'n/a')),
            ]),
        ])
    ])

  def CallToAction(self, record):
    """Call to action to use generated environment variables.

    Args:
      record: dict, the integration.

    Returns:
      A formatted string of the call to action message,
      or None if no call to action is required.
    """
    ## TODO(b/222759433):Once more than one redis instance is supported print
    ## correct variables. This will not be trivial since binding is not
    ## contained with redis resource.

    return ('To connect to the Redis instance utilize the '
            'environment variables {} and {}. These have '
            'been added to the Cloud Run service for you.'.format(
                'REDISHOST', 'REDISPORT'))

  def _VpcFromResources(self, records):
    for rec in records:
      if rec.get('type', None) == _VPC_INSTANCE_TYPE:
        return rec

    return {}

  def _RedisFromResources(self, records):
    for rec in records:
      if rec.get('type', None) == _REDIS_INSTANCE_TYPE:
        return rec

    return {}



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for storage commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import properties


# pylint:disable=invalid-name
def SetBucket(resource_ref, namespace, request):
  """Helper used in a declarative hook to set the bucket in a storage request.

  This is needed because the buckets resource is not rooted under a project,
  but a project is required when creating a bucket or listing buckets.

  Args:
    resource_ref: The parsed bucket resource
    namespace: unused
    request: The request the declarative framework has generated.

  Returns:
    The request to issue.
  """
  del namespace
  request.project = properties.VALUES.core.project.Get(required=True)
  request.bucket.name = resource_ref.bucket
  return request
# pylint:enable=invalid-name



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Base class for Spark Sql Job."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.dataproc.jobs import base as job_base
from googlecloudsdk.command_lib.dataproc.jobs import util as job_util


class SparkSqlBase(job_base.JobBase):
  """Submit a Spark SQL job to a cluster."""

  @staticmethod
  def Args(parser):
    """Parses command-line arguments specific to submitting SparkSql jobs."""
    driver = parser.add_mutually_exclusive_group(required=True)
    driver.add_argument(
        '--execute', '-e',
        metavar='QUERY',
        dest='queries',
        action='append',
        default=[],
        help='A Spark SQL query to execute as part of the job.')
    driver.add_argument(
        '--file', '-f',
        help=('HCFS URI of file containing Spark SQL script to execute as '
              'the job.'))
    parser.add_argument(
        '--jars',
        type=arg_parsers.ArgList(),
        metavar='JAR',
        default=[],
        help=('Comma separated list of jar files to be provided to the '
              'executor and driver classpaths. May contain UDFs.'))
    parser.add_argument(
        '--params',
        type=arg_parsers.ArgDict(),
        metavar='PARAM=VALUE',
        help='A list of key value pairs to set variables in the Hive queries.')
    parser.add_argument(
        '--properties',
        type=arg_parsers.ArgDict(),
        metavar='PROPERTY=VALUE',
        help='A list of key value pairs to configure Hive.')
    parser.add_argument(
        '--properties-file',
        help=job_util.PROPERTIES_FILE_HELP_TEXT)
    parser.add_argument(
        '--driver-log-levels',
        type=arg_parsers.ArgDict(),
        metavar='PACKAGE=LEVEL',
        help=('A list of package to log4j log level pairs to configure driver '
              'logging. For example: root=FATAL,com.example=INFO'))

  @staticmethod
  def GetFilesByType(args):
    return {
        'jars': args.jars,
        'file': args.file}

  @staticmethod
  def ConfigureJob(messages, job, files_by_type, logging_config, args):
    """Populates the sparkSqlJob member of the given job."""

    spark_sql_job = messages.SparkSqlJob(
        jarFileUris=files_by_type['jars'],
        queryFileUri=files_by_type['file'],
        loggingConfig=logging_config)

    if args.queries:
      spark_sql_job.queryList = messages.QueryList(queries=args.queries)
    if args.params:
      spark_sql_job.scriptVariables = encoding.DictToAdditionalPropertyMessage(
          args.params, messages.SparkSqlJob.ScriptVariablesValue)

    job_properties = job_util.BuildJobProperties(
        args.properties, args.properties_file)
    if job_properties:
    # Sort properties to ensure tests comparing messages not fail on ordering.
      spark_sql_job.properties = encoding.DictToAdditionalPropertyMessage(
          job_properties, messages.SparkSqlJob.PropertiesValue, sort_items=True)

    job.sparkSqlJob = spark_sql_job



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""VMware Engine VPC network peering routes list command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.vmware.networkpeeringroutes import NetworkPeeringRoutesClient
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.vmware.networks import flags
from googlecloudsdk.core.resource import resource_projector

DETAILED_HELP = {
    'DESCRIPTION':
        """
          List VPC network peering routes across all locations in your project.

        """,
    'EXAMPLES':
        """
          To list peering routes imported from peer network via my-peering:

            $ {command} --network-peering=my-peering --filter="direction=INCOMING"

          To list peering routes exported to peer network via my-peering:

            $ {command} --network-peering=my-peering --filter="direction=OUTGOING"

          In above examples, the location is taken as ``global''.
    """,
}


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List Google Cloud VMware Engine VPC network peering routes."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddNetworkPeeringToParser(parser)
    parser.display_info.AddFormat("""\
        table(
            dest_range,
            type,
            next_hop_region,
            status,
            direction)
    """)

  def Run(self, args):
    networkpeering = args.CONCEPTS.network_peering.Parse()
    client = NetworkPeeringRoutesClient()
    items = client.List(networkpeering)

    def _TransformStatus(direction, imported):
      """Create customized status field based on direction and imported."""
      if imported:
        if direction == 'INCOMING':
          return 'accepted'
        return 'accepted by peer'
      if direction == 'INCOMING':
        return 'rejected by config'
      return 'rejected by peer config'

    for item in items:
      route = resource_projector.MakeSerializable(item)
      # Set "status" to "Imported" or "Imported by peer" based on direction.
      route['status'] = _TransformStatus(route['direction'], route['imported'])
      yield route



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""gcloud dns response-policies rules update command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dns import util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dns import flags
from googlecloudsdk.command_lib.dns import resource_args
from googlecloudsdk.command_lib.dns import util as command_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


def _AddArgsCommon(parser):
  """Adds the common arguments for all versions."""
  flags.GetLocalDataResourceRecordSets().AddToParser(parser)
  # TODO(b/215745011) use AddResponsePolicyRulesBehaviorFlag once switch to v2
  flags.GetResponsePolicyRulesBehavior().AddToParser(parser)
  flags.GetLocationArg().AddToParser(parser)

  parser.add_argument(
      '--dns-name',
      required=False,
      help='DNS name (wildcard or exact) to apply this rule to.')


def _FetchResponsePolicyRule(response_policy, response_policy_rule, api_version,
                             args):
  """Get response policy rule to be Updated."""
  client = util.GetApiClient(api_version)
  m = apis.GetMessagesModule('dns', api_version)
  get_request = m.DnsResponsePolicyRulesGetRequest(
      responsePolicy=response_policy,
      project=properties.VALUES.core.project.Get(),
      responsePolicyRule=response_policy_rule)
  if api_version == 'v2':
    get_request.location = args.location
  return client.responsePolicyRules.Get(get_request)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Update(base.UpdateCommand):
  r"""Updates a new Cloud DNS response policy rule.

      This command updates a new Cloud DNS response policy rule.

      ## EXAMPLES

      To update a new response policy rule with DNS name, run:

        $ {command} myresponsepolicyrule --response-policy="myresponsepolicy"
        --dns-name="www.newzone.com." # pylint: disable=line-too-long

      To update a new response policy rule with local data rrsets, run:

        $ {command} myresponsepolicyrule --response-policy="myresponsepolicy"
        --local-data=name=www.zone.com.,type=A,ttl=21600,rrdatas=1.2.3.4

      To update a new response policy rule with behavior, run:

        $ {command} myresponsepolicyrule --response-policy="myresponsepolicy"
        --behavior=bypassResponsePolicy
  """

  @classmethod
  def _BetaOrAlpha(cls):
    return cls.ReleaseTrack() in (base.ReleaseTrack.BETA,
                                  base.ReleaseTrack.ALPHA)

  @classmethod
  def Args(cls, parser):
    _AddArgsCommon(parser)
    resource_args.AddResponsePolicyRuleArg(
        parser, verb='to update', api_version='v1')
    parser.display_info.AddFormat('json')

  def Run(self, args):
    api_version = util.GetApiFromTrackAndArgs(self.ReleaseTrack(), args)
    client = util.GetApiClient(api_version)
    messages = apis.GetMessagesModule('dns', api_version)

    # Get Response Policy Rule
    response_policy_rule_ref = args.CONCEPTS.response_policy_rule.Parse()
    response_policy_rule_name = response_policy_rule_ref.Name()

    response_policy_rule = messages.ResponsePolicyRule(
        ruleName=response_policy_rule_name)

    response_policy = messages.ResponsePolicy(
        responsePolicyName=args.response_policy)

    to_update = _FetchResponsePolicyRule(response_policy.responsePolicyName,
                                         response_policy_rule.ruleName,
                                         api_version, args)

    if not args.IsSpecified('dns_name') and not args.IsSpecified(
        'local_data') and not args.IsSpecified('behavior'):
      log.status.Print('Nothing to update.')
      return to_update

    if args.IsSpecified('dns_name'):
      to_update.dnsName = args.dns_name

    if args.IsSpecified('local_data'):
      to_update.behavior = None
      rrsets = []
      for rrset in args.local_data:
        resource_record_set = messages.ResourceRecordSet(
            name=rrset.get('name'),
            type=rrset.get('type'),
            ttl=rrset.get('ttl'),
            rrdatas=rrset.get('rrdatas').split('|'))
        rrsets.append(resource_record_set)
      to_update.localData = messages.ResponsePolicyRuleLocalData(
          localDatas=rrsets)

    if args.IsSpecified('behavior'):
      to_update.localData = None
      to_update.behavior = command_util.ParseResponsePolicyRulesBehavior(
          args, api_version)

    update_req = messages.DnsResponsePolicyRulesUpdateRequest(
        responsePolicy=response_policy.responsePolicyName,
        responsePolicyRule=response_policy_rule.ruleName,
        responsePolicyRuleResource=to_update,
        project=properties.VALUES.core.project.Get())

    if api_version == 'v2':
      update_req.location = args.location

    updated_response_policy_rule = client.responsePolicyRules.Update(update_req)

    log.UpdatedResource(updated_response_policy_rule.responsePolicyRule,
                        kind='ResponsePolicyRule')

    return updated_response_policy_rule



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating VM instances running Docker images."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import containers_utils
from googlecloudsdk.api_lib.compute import image_utils
from googlecloudsdk.api_lib.compute import instance_utils
from googlecloudsdk.api_lib.compute import metadata_utils
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.api_lib.compute.instances.create import utils as create_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute import completers
from googlecloudsdk.command_lib.compute import scope as compute_scopes
from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


def _Args(parser,
          deprecate_maintenance_policy=False,
          container_mount_enabled=False,
          support_multi_writer=True,
          support_confidential_compute_type=False):
  """Add flags shared by all release tracks."""
  parser.display_info.AddFormat(instances_flags.DEFAULT_LIST_FORMAT)
  metadata_utils.AddMetadataArgs(parser)
  instances_flags.AddDiskArgs(
      parser, True, container_mount_enabled=container_mount_enabled)
  instances_flags.AddCreateDiskArgs(
      parser,
      container_mount_enabled=container_mount_enabled,
      support_multi_writer=support_multi_writer)
  instances_flags.AddCanIpForwardArgs(parser)
  instances_flags.AddContainerMountDiskFlag(parser)
  instances_flags.AddAddressArgs(parser, instances=True, containers=True)
  instances_flags.AddAcceleratorArgs(parser)
  instances_flags.AddMachineTypeArgs(parser)
  instances_flags.AddMaintenancePolicyArgs(
      parser, deprecate=deprecate_maintenance_policy)
  instances_flags.AddNoRestartOnFailureArgs(parser)
  instances_flags.AddPreemptibleVmArgs(parser)
  instances_flags.AddProvisioningModelVmArgs(parser)
  instances_flags.AddInstanceTerminationActionVmArgs(parser)
  instances_flags.AddServiceAccountAndScopeArgs(parser, False)
  instances_flags.AddTagsArgs(parser)
  instances_flags.AddCustomMachineTypeArgs(parser)
  instances_flags.AddNetworkArgs(parser)
  instances_flags.AddPrivateNetworkIpArgs(parser)
  instances_flags.AddNetworkPerformanceConfigsArgs(parser)
  instances_flags.AddShieldedInstanceConfigArgs(
      parser=parser, for_container=True)
  instances_flags.AddKonletArgs(parser)
  instances_flags.AddPublicPtrArgs(parser, instance=True)
  instances_flags.AddImageArgs(parser)
  instances_flags.AddConfidentialComputeArgs(parser,
                                             support_confidential_compute_type)
  instances_flags.AddNestedVirtualizationArgs(parser)
  instances_flags.AddThreadsPerCoreArgs(parser)
  labels_util.AddCreateLabelsFlags(parser)

  parser.add_argument(
      '--description', help='Specifies a textual description of the instances.')

  instances_flags.INSTANCES_ARG.AddArgument(parser, operation_type='create')

  CreateWithContainer.SOURCE_INSTANCE_TEMPLATE = (
      instances_flags.MakeSourceInstanceTemplateArg())
  CreateWithContainer.SOURCE_INSTANCE_TEMPLATE.AddArgument(parser)
  parser.display_info.AddCacheUpdater(completers.InstancesCompleter)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class CreateWithContainer(base.CreateCommand):
  """Command for creating VM instances running container images."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = False
  _support_host_error_timeout_seconds = False
  _support_numa_node_count = False
  _support_visible_core_count = True
  _support_confidential_compute_type = False

  @staticmethod
  def Args(parser):
    """Register parser args."""
    _Args(parser, container_mount_enabled=True, support_multi_writer=False)
    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.GA)
    instances_flags.AddPrivateIpv6GoogleAccessArg(parser,
                                                  utils.COMPUTE_GA_API_VERSION)
    instances_flags.AddVisibleCoreCountArgs(parser)

  def _ValidateArgs(self, args):
    self._ValidateTrackSpecificArgs(args)
    instances_flags.ValidateAcceleratorArgs(args)
    instances_flags.ValidateNicFlags(args)
    instances_flags.ValidateNetworkTierArgs(args)
    instances_flags.ValidateKonletArgs(args)
    instances_flags.ValidateDiskCommonFlags(args)
    instances_flags.ValidateServiceAccountAndScopeArgs(args)
    instances_flags.ValidatePublicPtrFlags(args)
    instances_flags.ValidateNetworkPerformanceConfigsArgs(args)
    instances_flags.ValidateInstanceScheduling(args)
    if instance_utils.UseExistingBootDisk(args.disk or []):
      raise exceptions.InvalidArgumentException(
          '--disk', 'Boot disk specified for containerized VM.')

  def _ValidateTrackSpecificArgs(self, args):
    return None

  def GetImageUri(self, args, compute_client, resource_parser, instance_refs):
    if (args.IsSpecified('image') or args.IsSpecified('image_family') or
        args.IsSpecified('image_project')):
      image_expander = image_utils.ImageExpander(compute_client,
                                                 resource_parser)
      image_uri, _ = image_expander.ExpandImageFlag(
          user_project=instance_refs[0].project,
          image=args.image,
          image_family=args.image_family,
          image_project=args.image_project)
      if resource_parser.Parse(image_uri).project != 'cos-cloud':
        log.warning('This container deployment mechanism requires a '
                    'Container-Optimized OS image in order to work. Select an '
                    'image from a cos-cloud project (cos-stable, cos-beta, '
                    'cos-dev image families).')
    else:
      image_uri = containers_utils.ExpandKonletCosImageFlag(compute_client)
    return image_uri

  def _GetNetworkInterfaces(self, args, client, holder, project, location,
                            scope, skip_defaults):
    return create_utils.GetNetworkInterfaces(args, client, holder, project,
                                             location, scope, skip_defaults)

  def GetNetworkInterfaces(self, args, resources, client, holder, project,
                           location, scope, skip_defaults):
    if args.network_interface:
      return create_utils.CreateNetworkInterfaceMessages(
          resources=resources,
          compute_client=client,
          network_interface_arg=args.network_interface,
          project=project,
          location=location,
          scope=scope)
    return self._GetNetworkInterfaces(args, client, holder, project, location,
                                      scope, skip_defaults)

  def CheckDiskMessageArgs(self, args, skip_defaults):
    """Creates API messages with disks attached to VM instance."""
    flags_to_check = [
        'create_disk', 'local_ssd', 'boot_disk_type', 'boot_disk_device_name',
        'boot_disk_auto_delete', 'boot_disk_provisioned_iops'
    ]
    if hasattr(args, 'local_nvdimm'):
      flags_to_check.append('local_nvdimm')
    if (skip_defaults and not args.IsSpecified('disk') and
        not instance_utils.IsAnySpecified(args, *flags_to_check)):
      return False
    return True

  def Run(self, args):
    self._ValidateArgs(args)

    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    compute_client = holder.client
    resource_parser = holder.resources

    container_mount_disk = instances_flags.GetValidatedContainerMountDisk(
        holder, args.container_mount_disk, args.disk, args.create_disk)
    source_instance_template = instance_utils.GetSourceInstanceTemplate(
        args, resource_parser, self.SOURCE_INSTANCE_TEMPLATE)
    skip_defaults = instance_utils.GetSkipDefaults(source_instance_template)
    scheduling = instance_utils.GetScheduling(
        args,
        compute_client,
        skip_defaults,
        support_min_node_cpu=False,
        support_host_error_timeout_seconds=self
        ._support_host_error_timeout_seconds)
    service_accounts = instance_utils.GetServiceAccounts(
        args, compute_client, skip_defaults)
    user_metadata = instance_utils.GetValidatedMetadata(args, compute_client)
    boot_disk_size_gb = instance_utils.GetBootDiskSizeGb(args)
    instance_refs = instance_utils.GetInstanceRefs(args, compute_client, holder)
    network_interfaces = self.GetNetworkInterfaces(
        args, resource_parser, compute_client, holder, instance_refs[0].project,
        instance_refs[0].zone, compute_scopes.ScopeEnum.ZONE, skip_defaults)
    image_uri = self.GetImageUri(args, compute_client, resource_parser,
                                 instance_refs)
    labels = containers_utils.GetLabelsMessageWithCosVersion(
        args.labels, image_uri, resource_parser,
        compute_client.messages.Instance)
    can_ip_forward = instance_utils.GetCanIpForward(args, skip_defaults)
    tags = containers_utils.CreateTagsMessage(compute_client.messages,
                                              args.tags)
    confidential_vm = (
        args.IsSpecified('confidential_compute') and args.confidential_compute)

    if self._support_confidential_compute_type:
      confidential_vm |= (
          args.IsSpecified('confidential_compute_type') and
          args.confidential_compute_type is not None)

    requests = []
    for instance_ref in instance_refs:
      metadata = containers_utils.CreateKonletMetadataMessage(
          compute_client.messages,
          args,
          instance_ref.Name(),
          user_metadata,
          container_mount_disk_enabled=self._container_mount_disk_enabled,
          container_mount_disk=container_mount_disk)

      disks = []
      if self.CheckDiskMessageArgs(args, skip_defaults):
        disks = create_utils.CreateDiskMessages(
            args=args,
            instance_name=instance_ref.Name(),
            project=instance_ref.project,
            location=instance_ref.zone,
            scope=compute_scopes.ScopeEnum.ZONE,
            compute_client=compute_client,
            resource_parser=resource_parser,
            boot_disk_size_gb=boot_disk_size_gb,
            image_uri=image_uri,
            create_boot_disk=self._support_create_boot_disk,
            support_nvdimm=self._support_nvdimm,
            support_match_container_mount_disks=self
            ._support_match_container_mount_disks)

      machine_type_uri = None
      if instance_utils.CheckSpecifiedMachineTypeArgs(args, skip_defaults):
        machine_type_uri = create_utils.CreateMachineTypeUri(
            args=args,
            compute_client=compute_client,
            resource_parser=resource_parser,
            project=instance_ref.project,
            location=instance_ref.zone,
            scope=compute_scopes.ScopeEnum.ZONE,
            confidential_vm=confidential_vm)

      guest_accelerators = create_utils.GetAccelerators(
          args=args,
          compute_client=compute_client,
          resource_parser=resource_parser,
          project=instance_ref.project,
          location=instance_ref.zone,
          scope=compute_scopes.ScopeEnum.ZONE)

      instance = compute_client.messages.Instance(
          canIpForward=can_ip_forward,
          disks=disks,
          guestAccelerators=guest_accelerators,
          description=args.description,
          labels=labels,
          machineType=machine_type_uri,
          metadata=metadata,
          minCpuPlatform=args.min_cpu_platform,
          name=instance_ref.Name(),
          networkInterfaces=network_interfaces,
          serviceAccounts=service_accounts,
          scheduling=scheduling,
          tags=tags)
      if args.private_ipv6_google_access_type is not None:
        instance.privateIpv6GoogleAccess = (
            instances_flags.GetPrivateIpv6GoogleAccessTypeFlagMapper(
                compute_client.messages).GetEnumForChoice(
                    args.private_ipv6_google_access_type))

      confidential_instance_config = (
          create_utils.BuildConfidentialInstanceConfigMessage(
              messages=compute_client.messages,
              args=args,
              support_confidential_compute_type=self
              ._support_confidential_compute_type))
      if confidential_instance_config:
        instance.confidentialInstanceConfig = confidential_instance_config

      has_visible_core_count = (
          self._support_visible_core_count and
          args.visible_core_count is not None)
      if (args.enable_nested_virtualization is not None or
          args.threads_per_core is not None or
          (self._support_numa_node_count and
           args.numa_node_count is not None) or has_visible_core_count):
        visible_core_count = args.visible_core_count if has_visible_core_count else None
        instance.advancedMachineFeatures = (
            instance_utils.CreateAdvancedMachineFeaturesMessage(
                compute_client.messages, args.enable_nested_virtualization,
                args.threads_per_core,
                args.numa_node_count if self._support_numa_node_count else None,
                visible_core_count))

      shielded_instance_config = create_utils.BuildShieldedInstanceConfigMessage(
          messages=compute_client.messages, args=args)
      if shielded_instance_config:
        instance.shieldedInstanceConfig = shielded_instance_config

      if args.IsSpecified('network_performance_configs'):
        instance.networkPerformanceConfig = (
            instance_utils.GetNetworkPerformanceConfig(args, compute_client))

      request = compute_client.messages.ComputeInstancesInsertRequest(
          instance=instance,
          sourceInstanceTemplate=source_instance_template,
          project=instance_ref.project,
          zone=instance_ref.zone)

      requests.append(
          (compute_client.apitools_client.instances, 'Insert', request))

    return compute_client.MakeRequests(requests)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateWithContainerBeta(CreateWithContainer):
  """Command for creating VM instances running container images."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = False
  _support_visible_core_count = True
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = False

  @staticmethod
  def Args(parser):
    """Register parser args."""
    _Args(parser, container_mount_enabled=True)
    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddLocalSsdArgs(parser)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.BETA)
    instances_flags.AddPrivateIpv6GoogleAccessArg(
        parser, utils.COMPUTE_BETA_API_VERSION)
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)
    instances_flags.AddVisibleCoreCountArgs(parser)

  def _ValidateTrackSpecificArgs(self, args):
    instances_flags.ValidateLocalSsdFlags(args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateWithContainerAlpha(CreateWithContainerBeta):
  """Alpha version of compute instances create-with-container command."""

  _container_mount_disk_enabled = True
  _support_create_boot_disk = True
  _support_match_container_mount_disks = True
  _support_nvdimm = True
  _support_host_error_timeout_seconds = True
  _support_numa_node_count = True
  _support_visible_core_count = True
  _support_confidential_compute_type = True

  @staticmethod
  def Args(parser):
    _Args(
        parser,
        deprecate_maintenance_policy=True,
        container_mount_enabled=True,
        support_confidential_compute_type=True)

    instances_flags.AddNetworkTierArgs(parser, instance=True)
    instances_flags.AddLocalSsdArgsWithSize(parser)
    instances_flags.AddLocalNvdimmArgs(parser)
    instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.ALPHA)
    instances_flags.AddPublicDnsArgs(parser, instance=True)
    instances_flags.AddPrivateIpv6GoogleAccessArg(
        parser, utils.COMPUTE_ALPHA_API_VERSION)
    instances_flags.AddStackTypeArgs(parser)
    instances_flags.AddIpv6NetworkTierArgs(parser)
    instances_flags.AddHostErrorTimeoutSecondsArgs(parser)
    instances_flags.AddNumaNodeCountArgs(parser)
    instances_flags.AddVisibleCoreCountArgs(parser)
    instances_flags.AddIPv6AddressArgs(parser)
    instances_flags.AddIPv6PrefixLengthArgs(parser)

  def _ValidateTrackSpecificArgs(self, args):
    instances_flags.ValidateLocalSsdFlags(args)
    instances_flags.ValidatePublicDnsFlags(args)
    instances_flags.ValidatePublicPtrFlags(args)

  def _GetNetworkInterfaces(self, args, client, holder, project, location,
                            scope, skip_defaults):
    return create_utils.GetNetworkInterfacesAlpha(args, client, holder, project,
                                                  location, scope,
                                                  skip_defaults)


CreateWithContainer.detailed_help = {
    'brief':
        """\
    Creates Compute Engine virtual machine instances running
    container images.
    """,
    'DESCRIPTION':
        """\
        *{command}* creates Compute Engine virtual
        machines that runs a Docker image. For example:

          $ {command} instance-1 --zone us-central1-a \
            --container-image=gcr.io/google-containers/busybox

        creates an instance called instance-1, in the us-central1-a zone,
        running the 'busybox' image.

        For more examples, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES':
        """\
        To run the gcr.io/google-containers/busybox image on an instance named
        'instance-1' that executes 'echo "Hello world"' as a run command, run:

          $ {command} instance-1 \
            --container-image=gcr.io/google-containers/busybox \
            --container-command='echo "Hello world"'

        To run the gcr.io/google-containers/busybox image in privileged mode,
        run:

          $ {command} instance-1 \
            --container-image=gcr.io/google-containers/busybox
            --container-privileged
        """
}



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Convenience utilities for building python calliope config export commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.util.declarative import flags as declarative_config_flags
from googlecloudsdk.command_lib.util.declarative.clients import kcc_client


def BuildHelpText(singular, plural=None, service=None, begins_with_vowel=False):
  """Builds and returns help text for declarative export commands."""

  # If plural not specified, just add an 's'.
  plural = plural or '{}s'.format(singular)

  # First mentions should include service name if provided.
  singular_with_service = singular
  if service:
    singular_with_service = '{} {}'.format(service, singular)

  # Determine if resource starts with a vowel and use 'a' or 'an' respectively.
  a_or_an = 'a'
  if begins_with_vowel:
    a_or_an = 'an'

  resource_name = '-'.join(singular.lower().split())

  detailed_help = {
      'brief':
          'Export the configuration for {a_or_an} {singular_with_service}.'
          .format(a_or_an=a_or_an, singular_with_service=singular_with_service),
      'DESCRIPTION':
          """\
            *{{command}}* exports the configuration for {a_or_an} {singular_with_service}.

            {singular_capitalized} configurations can be exported in
            Kubernetes Resource Model (krm) or Terraform HCL formats. The
            default format is `krm`.

            Specifying `--all` allows you to export the configurations for all
            {plural} within the project.

            Specifying `--path` allows you to export the configuration(s) to
            a local directory.
          """.format(
              singular_capitalized=singular.capitalize(),
              singular_with_service=singular_with_service,
              plural=plural,
              a_or_an=a_or_an),
      'EXAMPLES':
          """\
            To export the configuration for {a_or_an} {singular}, run:

              $ {{command}} my-{resource_name}

            To export the configuration for {a_or_an} {singular} to a file, run:

              $ {{command}} my-{resource_name} --path=/path/to/dir/

            To export the configuration for {a_or_an} {singular} in Terraform
            HCL format, run:

              $ {{command}} my-{resource_name} --resource-format=terraform

            To export the configurations for all {plural} within a
            project, run:

              $ {{command}} --all
          """.format(
              singular=singular,
              plural=plural,
              resource_name=resource_name,
              a_or_an=a_or_an)
  }
  return detailed_help


def RegisterArgs(parser, add_to_parser, **kwargs):
  mutex_group = parser.add_group(mutex=True, required=True)
  resource_group = mutex_group.add_group()
  add_to_parser(resource_group, **kwargs)
  declarative_config_flags.AddAllFlag(mutex_group, collection='project')
  declarative_config_flags.AddPathFlag(parser)
  declarative_config_flags.AddFormatFlag(parser)


def RunExport(args, collection, resource_ref):
  client = kcc_client.KccClient()
  if getattr(args, 'all', None):
    return client.ExportAll(args=args, collection=collection)
  return client.Export(args, resource_uri=resource_ref)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Custom domain formatter for Cloud Run Integrations."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from googlecloudsdk.command_lib.run.integrations.formatters import base_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import states
from googlecloudsdk.core.console import console_attr
from googlecloudsdk.core.resource import custom_printer_base as cp


class CustomDomainFormatter(base_formatter.BaseFormatter):
  """Format logics for custom domain integration."""

  def TransformConfig(self, record):
    """Print the config of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    res_config = record.get('config', {}).get('router', {})
    labeled = [
        ('Domain', res_config.get('domain', '')),
    ]
    if 'default-route' in res_config:
      labeled.append(('Path', '/*'))
      labeled.append(
          ('Service',
           self._GetServiceName(res_config.get('default-route',
                                               {}).get('ref', ''))))
    for r in res_config.get('routes', []):
      labeled.append(('Path', ','.join(r.get('paths', ''))))
      labeled.append(('Service', self._GetServiceName(r.get('ref', ''))))
    return cp.Labeled(labeled)

  def TransformComponentStatus(self, record):
    """Print the component status of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    resource_status = record.get('status', {})
    resource_components = resource_status.get('resourceComponentStatuses', {})
    details = resource_status.get('routerDetails', {})
    return cp.Labeled([
        cp.Lines([
            ('Google Cloud Load Balancer ({})'.format(
                self._GetGCLBName(resource_components))),
            cp.Labeled([
                ('Console link', resource_status.get('consoleLink', 'n/a')),
                ('Frontend', details.get('ipAddress', 'n/a')),
                ('SSL Certificate',
                 self.PrintStatus(self._GetSSLStatus(resource_components))),
            ]),
        ])
    ])

  def CallToAction(self, record):
    """Call to action to configure IP for the domain.

    Args:
      record: dict, the integration.

    Returns:
      A formatted string of the call to action message,
      or None if no call to action is required.
    """
    resource_config = record.get('config')
    resource_status = record.get('status')
    if not resource_status or not resource_status:
      return None

    domain = resource_config.get('router', {}).get('domain')
    resource_components = resource_status.get('resourceComponentStatuses', {})
    ssl_status = self._GetSSLStatus(resource_components)
    ip = resource_status.get('routerDetails', {}).get('ipAddress')
    if domain and ip and ssl_status != states.ACTIVE:
      domain_padding = len(domain) - len('NAME')
      padding_string = ' ' * domain_padding
      con = console_attr.GetConsoleAttr()
      return ('{0} To complete the process, please ensure the following '
              'DNS records are configured for the domain "{1}":\n'
              '    NAME{2}  TTL   TYPE  DATA\n'
              '    {1}  3600  A     {3}\n'
              'It can take up to an hour for the certificate to be provisioned.'
              .format(con.Colorize('!', 'yellow'), domain, padding_string, ip))
    return None

  def _GetServiceName(self, ref):
    parts = ref.split('/')
    if len(parts) == 2 and parts[0] == 'service':
      ref = parts[1]
    return ref

  def _GetGCLBName(self, resource_components):
    url_map = self._FindResourceByType(resource_components,
                                       'google_compute_url_map')
    if url_map:
      return url_map.get('name', 'n/a')
    return 'n/a'

  def _GetSSLStatus(self, resource_components):
    ssl_cert = self._FindResourceByType(
        resource_components, 'google_compute_managed_ssl_certificate')
    if ssl_cert:
      return ssl_cert.get('state', states.UNKNOWN)
    return states.UNKNOWN

  def _FindResourceByType(self, resources, rtype):
    if not resources:
      return None
    for resource in resources:
      if resource.get('type') == rtype:
        return resource



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for OS Config commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


def PatchJobAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='patch_job', help_text='An OS patch job.')


def GetPatchJobResourceSpec():
  return concepts.ResourceSpec(
      'osconfig.projects.patchJobs',
      resource_name='patch_job',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      patchJobsId=PatchJobAttributeConfig())


def CreatePatchJobResourceArg(verb, plural=False):
  """Creates a resource argument for a OS Config patch job.

  Args:
    verb: str, The verb to describe the resource, such as 'to describe'.
    plural: bool, If True, use a resource argument that returns a list.

  Returns:
    PresentationSpec for the resource argument.
  """
  noun = 'Patch job' + ('s' if plural else '')
  return presentation_specs.ResourcePresentationSpec(
      'patch_job',
      GetPatchJobResourceSpec(),
      '{} {}'.format(noun, verb),
      required=True,
      plural=plural,
      prefixes=False)


def AddPatchJobResourceArg(parser, verb, plural=False):
  """Creates a resource argument for a OS Config patch job.

  Args:
    parser: The parser for the command.
    verb: str, The verb to describe the resource, such as 'to describe'.
    plural: bool, If True, use a resource argument that returns a list.
  """
  concept_parsers.ConceptParser([CreatePatchJobResourceArg(
      verb, plural)]).AddToParser(parser)


def PatchDeploymentAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='patch_deployment', help_text='An OS patch deployment.')


def GetPatchDeploymentResourceSpec():
  return concepts.ResourceSpec(
      'osconfig.projects.patchDeployments',
      resource_name='patch_deployment',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      patchDeploymentsId=PatchDeploymentAttributeConfig())


def CreatePatchDeploymentResourceArg(verb, plural=False):
  """Creates a resource argument for a OS Config patch deployment.

  Args:
    verb: str, The verb to describe the resource, such as 'to describe'.
    plural: bool, If True, use a resource argument that returns a list.

  Returns:
    PresentationSpec for the resource argument.
  """
  noun = 'Patch deployment' + ('s' if plural else '')
  return presentation_specs.ResourcePresentationSpec(
      'patch_deployment',
      GetPatchDeploymentResourceSpec(),
      '{} {}'.format(noun, verb),
      required=True,
      plural=plural,
      prefixes=False)


def AddPatchDeploymentResourceArg(parser, verb, plural=False):
  """Creates a resource argument for a OS Config patch deployment.

  Args:
    parser: The parser for the command.
    verb: str, The verb to describe the resource, such as 'to describe'.
    plural: bool, If True, use a resource argument that returns a list.
  """
  concept_parsers.ConceptParser(
      [CreatePatchDeploymentResourceArg(verb, plural)]).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of objects get-iam-policy command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class GetIamPolicy(base.Command):
  """Get the access policy for an object."""

  detailed_help = {
      'DESCRIPTION':
          """
      *{command}* behaves similarly to *{parent_command} get-object-acl*, but
      uses the IAM policy binding syntax in the output.
      """,
      'EXAMPLES':
          """
      To get the access policy for OBJECT-1 in BUCKET-1:

        $ {command} gs://BUCKET-1/OBJECT-1

      To output the access policy for OBJECT-1 in BUCKET-1 to a file:

        $ {command} gs://BUCKET-1/OBJECT-1 > policy.txt
      """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'url',
        nargs='+',
        help='URLs for objects whose access policies are being requested.')

  def Run(self, args):
    del args  # Unused.
    raise NotImplementedError



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generate random names."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import random
import string

# The strings below contain bad words, swear words, and otherwise offensive
# words as well as strings phonetically similar to offensive words and
# misspellings of offensive words.
# This list is used as a blocklist to prevent potentially offensive words
# from being generated and shown to users. While offensive words may
# still be generated from combining multiple 3-letter words, this blocklist is
# a best effort to limit any potential offensiveness. Words may be added or
# removed from this list as deemed appropriate.
#
# Only 3-letter words matching the consonant-vowel-consonant pattern are
# included.
# TODO(b/183436507) Sync the list
_INVALID_THREE_LETTER_WORDS = frozenset({
    'bah',
    'baj',
    'bal',
    'bam',
    'bar',
    'beh',
    'bew',
    'bez',
    'bic',
    'bin',
    'bod',
    'bok',
    'bol',
    'bon',
    'bow',
    'box',
    'bun',
    'bur',
    'bus',
    'cac',
    'cak',
    'caq',
    'cin',
    'coc',
    'cok',
    'con',
    'coq',
    'coz',
    'cuk',
    'cul',
    'cum',
    'cun',
    'cur',
    'dan',
    'daw',
    'day',
    'dem',
    'dev',
    'dic',
    'dik',
    'diq',
    'dix',
    'dom',
    'dot',
    'dud',
    'fag',
    'fak',
    'fan',
    'fap',
    'fas',
    'fek',
    'fel',
    'fez',
    'fis',
    'fob',
    'fok',
    'fot',
    'fuc',
    'fuk',
    'fuq',
    'fut',
    'fux',
    'gad',
    'gal',
    'gan',
    'gar',
    'gat',
    'gay',
    'gec',
    'gey',
    'gid',
    'gil',
    'giz',
    'gog',
    'gop',
    'got',
    'goy',
    'guc',
    'gud',
    'guk',
    'guq',
    'hag',
    'hah',
    'heh',
    'hen',
    'het',
    'hon',
    'hor',
    'huj',
    'hul',
    'hur',
    'hus',
    'jap',
    'jav',
    'jeb',
    'jew',
    'jit',
    'jiz',
    'job',
    'kac',
    'kak',
    'kan',
    'kaq',
    'kar',
    'kaw',
    'kef',
    'kel',
    'ken',
    'kep',
    'kik',
    'kir',
    'koc',
    'kok',
    'koq',
    'kor',
    'kox',
    'kuk',
    'kum',
    'kun',
    'kus',
    'kut',
    'kuz',
    'lam',
    'lan',
    'las',
    'lem',
    'let',
    'lic',
    'lid',
    'lik',
    'lil',
    'liq',
    'lol',
    'lop',
    'lox',
    'lud',
    'lul',
    'lun',
    'lus',
    'luz',
    'maj',
    'mal',
    'meg',
    'meh',
    'mes',
    'mic',
    'mik',
    'min',
    'miq',
    'mor',
    'muf',
    'mul',
    'mun',
    'mut',
    'nad',
    'nas',
    'nib',
    'nig',
    'nim',
    'noq',
    'nun',
    'nut',
    'pad',
    'pah',
    'pap',
    'par',
    'pat',
    'paz',
    'pek',
    'pel',
    'pes',
    'pik',
    'pis',
    'pix',
    'pod',
    'pom',
    'pot',
    'pug',
    'puk',
    'pum',
    'pus',
    'qab',
    'qij',
    'quz',
    'rac',
    'rak',
    'ral',
    'ran',
    'raq',
    'rev',
    'ris',
    'rit',
    'rot',
    'ruc',
    'sac',
    'sak',
    'saq',
    'sat',
    'sek',
    'ser',
    'set',
    'sex',
    'sey',
    'sik',
    'sob',
    'sod',
    'sol',
    'sot',
    'soy',
    'suc',
    'sud',
    'suk',
    'suq',
    'sut',
    'tal',
    'tay',
    'tem',
    'tin',
    'tit',
    'toc',
    'tog',
    'tok',
    'toq',
    'tos',
    'tun',
    'tup',
    'vag',
    'vaj',
    'wac',
    'wah',
    'wak',
    'waq',
    'war',
    'was',
    'wat',
    'wen',
    'wog',
    'wop',
    'xuy',
    'yal',
    'yid',
    'yor',
    'yuk',
    'zak',
    'zeb',
    'zig',
    'zov',
    'zut',
})

_VOWELS = 'aeiou'

_CONSONANTS = ''.join(l for l in string.ascii_lowercase if l not in _VOWELS)


def IsValidWord(word):
  return word not in _INVALID_THREE_LETTER_WORDS


def _ThreeLetterGenerator(validate):
  """Generate random 3-letter words.

  Words are generated in a consonant-vowel-consonant order to be pronounceable.
  A specific word matching this pattern has a 1/21*1/5*1/21 = 1/2205 chance
  of being generated.

  Args:
    validate: bool, True to validate words against the invalid set.

  Returns:
    str, 3-letter word
  """
  while True:
    word = (
        random.choice(_CONSONANTS) + random.choice(_VOWELS) +
        random.choice(_CONSONANTS))
    if not validate or IsValidWord(word):
      return word


def GenerateName(sections=1, separator='-', prefix=None, validate=True):
  """Generate a random string of 3-letter sections.

  Each word has a 1/2205 chance of being generated (see _GenerateThreeLetter).
  Therefore a specific name has a (1/2205)^(sections) chance of being generated.
  For 3 sections, the denominator is over 10.7 billion.

  Args:
    sections: int, number of 3-letter generated sections to include
    separator: str, separator between sections
    prefix: str, prefix of the generated name. This acts like an additional
      section at the start of the name and will be separated from the
      generated sections by the seperator argument, however it does not count
      towards the number of sections specified by the sections argument.
    validate: bool, True to validate sections against invalid word list

  Returns:
    str, generated name
  """
  assert sections > 0
  names = [_ThreeLetterGenerator(validate) for _ in range(sections)]
  if prefix is not None:
    names.insert(0, prefix)
  return separator.join(names)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the Cloud Filestore operations commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

OPERATIONS_LIST_FORMAT = """\
    table(
        name.basename():label=OPERATION_NAME,
        name.segment(3):label=LOCATION,
        metadata.verb:label=TYPE,
        metadata.target.basename(),
        done.yesno(yes='DONE', no='RUNNING'):label=STATUS,
        metadata.createTime.date():sort=1,
        duration(start=metadata.createTime,end=metadata.endTime,precision=0,calendar=false).slice(2:).join("").yesno(no="<1S"):label=DURATION
    )"""

